const char SSID[] = "";
const char PASSWORD[] = "";

const char version[] = " 0.8";
#include "globalvars.h"
#include <Arduino.h>
#include <EEPROM.h>
#include <ESP8266WebServer.h>
#include <ESP8266WiFi.h>
#include <ezTime.h>
#include <stdint.h>

ESP8266WebServer server;
IPAddress staticIP(192, 168, 2, 5); // static IP address
IPAddress gateway(192, 168, 2, 1);  // Router's IP address
IPAddress subnet(255, 255, 255, 0);
IPAddress dns(192, 168, 2, 1);
Timezone myTime;

char time_all[8][8][3] = {
    {}, {}, {}, {},
    {}, {}, {}, {}}; // 3D array for load_EEPROM() and handleTime()
const unsigned int pins[8] = {5,  4,  14, 12,
                              13, 10, 9}; // TODO: only 7 of 8 pins defined
int r = 0;

char stat[8][4] = {"AUS", "AUS", "AUS", "AUS",
                   "AUS", "AUS", "AUS", "AUS"}; // text on button
char t_stat[8][4] = {"", "", "", "",
                     "", "", "", ""}; // for "status of timer changed"
bool override[8] = {false, false, false, false,
                    false, false, false, false}; // for manual override

char home_html[3000]; // be careful when changing these values (might cause
                      // overflow)!
char settings_html[2014];
const char settings_html_pre[] PROGMEM = {"<!DOCTYPE html>\
<html lang='de'>\
    <head>\
        <meta charset='UTF-8'>\
        <meta http-equiv='X-UA-Compatible' content='IE=edge'>\
        <meta name='viewport' content='width=device-width, initial-scale=1.0'>\
        <title>Einstellungen</title>\
        <style>\
            * {\
                font-family: Arial, Helvetica, sans-serif;\
                font-size: 26px;\
                user-select: none;\
                text-decoration: none;\
            }\
            a {\
                color: black;\
            }\
            .wrapper_main {\
                margin: 0 auto;\
                max-width: fit-content;\
                max-width: -moz-fit-content;\
                text-align: center;\
                background-color: rgb(214, 214, 214);\
                padding: 1px 8px 8px 8px;\
                border-radius: 4px;\
            }\
            #rule, [type='time'] {\
                height: 36px;\
                border: none;\
                padding: 0 4px;\
                margin: 4px 0;\
                background-color: white;\
                width: fit-content;\
                width: -moz-fit-content;\
                border-radius: 4px;\
            }\
            .animated_a, [type='submit']{\
                background-color: rgb(135, 252, 119);\
                border: none;\
                border-radius: 4px;\
                box-shadow: rgb(25, 167, 49) 0 4px;\
                position: relative;\
                margin: 8px 0;\
                padding: 2px 8px;\
            }\
            [type='submit'] {\
                position: relative;\
                top: -2px;\
                display: inline-block;\
            }\
            .animated_a {\
                text-decoration: none;\
                display: inline-block;\
                margin: 8px 0;\
                padding-top: 4px;\
                height: 28px;\
            }\
            .wrapper_table {\
                overflow: auto;\
            }\
            table {\
                margin: 8px auto;\
            }\
            table, th, td {\
                border: 1px solid black;\
                border-collapse: collapse;\
                user-select: text;\
                padding: 0 8px;\
            }\
            .animated_a, [type='submit']{\
                cursor: pointer;\
            }\
            .animated_a:active {\
                box-shadow: rgb(25, 167, 49) 0 2px;\
                top: 2px;\
            }\
            [type='submit']:active {\
                box-shadow: rgb(25, 167, 49) 0 2px;\
                top: 0px;\
            }\
            *:focus {\
                outline: none;\
            }\
        </style>\
    </head>\
    <body>\
        <div class='wrapper_main'>\
            <form action='settings' method='POST'>\
                <select name='rule' id='rule' required>\
                    <option value='' selected disabled>Ventiel Nr.</option>\
                    <option value='0'>1</option>\
                    <option value='1'>2</option>\
                    <option value='2'>3</option>\
                    <option value='3'>4</option>\
                    <option value='4'>5</option>\
                    <option value='5'>6</option>\
                    <option value='6'>7</option>\
                    <option value='7' disabled>8</option>\
                </select>\
                <br>\
                <input type='time' name='time_o'>\
                - <input type='time' name='time_f'>\
                <br>\
                <input type='time' name='time_o_1'>\
                - <input type='time' name='time_f_1'>\
                <br>\
                <input type='submit'>\
            </form>\
            <div class='wrapper_table'>\
                <table>\
                    <tr>\
                        <th style='background-color: gray;'></th>\
                        <th>von:</th>\
                        <th>bis:</th>\
                        <th>von:</th>\
                        <th>bis:</th>\
                    </tr>"};
void reload_home();
void reload_settings();
void handleToggle();

void load_EEPROM() {
  unsigned int EEPROM_Addr = 0;
  for (unsigned int i0 = 0; i0 < 8; i0++) {
    for (unsigned int i = 0; i < 8; i++) {
      if (EEPROM.read(EEPROM_Addr) < 10) {
        sprintf(time_all[i0][i], "0%d", EEPROM.read(EEPROM_Addr));
      } else {
        sprintf(time_all[i0][i], "%d", EEPROM.read(EEPROM_Addr));
      }
      EEPROM_Addr += 4;
      yield();
    }
  }
}

void handleHome() {
  reload_home();
  server.send(200, "text/html", home_html);
  debugln(ESP.getFreeHeap(), DEC);
}

void handleSettings() {
  reload_settings();
  server.setContentLength(strlen(settings_html_pre) + strlen(settings_html));
  server.send(200, "text/html", settings_html_pre);
  server.sendContent(settings_html);
  debugln(ESP.getFreeHeap(), DEC);
}

void handleTime() {
  const char args[4][9] = {"time_o", "time_f", "time_o_1", "time_f_1"};

  for (unsigned int arg = 0; arg < 4; arg++) {
    if (server.arg(args[arg]) != "") {
      char buff_h[2][2] = {server.arg(args[arg])[0], server.arg(args[arg])[1]};
      char buff_m[2][2] = {server.arg(args[arg])[3], server.arg(args[arg])[4]};
      sprintf(time_all[arg * 2][server.arg("rule").toInt()], "%s", buff_h);
      sprintf(time_all[arg * 2 + 1][server.arg("rule").toInt()], "%s", buff_m);

      EEPROM.put(server.arg("rule").toInt() * 4 + arg * 64,
                 atoi(time_all[arg * 2][server.arg("rule").toInt()]));
      EEPROM.commit();
      EEPROM.put(server.arg("rule").toInt() * 4 + arg * 64 + 32,
                 atoi(time_all[arg * 2 + 1][server.arg("rule").toInt()]));
      EEPROM.commit();
    }
  }
  reload_settings();
  server.setContentLength(strlen(settings_html_pre) + strlen(settings_html));
  server.send(301, "text/html", settings_html_pre);
  server.sendContent(settings_html);
}

void timer() {
  if (((myTime.hour() * 60 + myTime.minute() >=
        atoi(time_all[0][r]) * 60 + atoi(time_all[1][r])) and
       (myTime.hour() * 60 + myTime.minute() <
        atoi(time_all[2][r]) * 60 + atoi(time_all[3][r]))) or
      ((myTime.hour() * 60 + myTime.minute() >=
        atoi(time_all[4][r]) * 60 + atoi(time_all[5][r])) and
       (myTime.hour() * 60 + myTime.minute() <
        atoi(time_all[6][r]) * 60 + atoi(time_all[7][r])))) {
    if ((override[r]) and (strcmp(t_stat[r], "off") == 0)) {
      override[r] = false;
      debugln(F("debug#1"));
    }
    sprintf(t_stat[r], "on");
    if (!override[r]) {
      digitalWrite(pins[r], LOW);
      if (strcmp(stat[r], "AUS") == 0) {
        debug(r);
        debugln(F(" is on"));
        sprintf(stat[r], "AN");
      }
    }
  } else {
    if ((override[r]) and (strcmp(t_stat[r], "on") == 0)) {
      override[r] = false;
    }
    sprintf(t_stat[r], "off");
    if (!override[r]) {
      digitalWrite(pins[r], HIGH);
      if (strcmp(stat[r], "AN") == 0) {
        debug(r);
        debugln(F(" is off"));
        sprintf(stat[r], "AUS");
      }
    }
  }
}

void setup() {
  int p = 0;
  while (p <= 7) {
    pinMode(pins[p], OUTPUT);
    digitalWrite(pins[p], HIGH);
    p++;
  }
  Serial.begin(74880);
  EEPROM.begin(256); // 4 bits for EEPROM, address 0-255, value 0-255
  debug(F("\nver."));
  debugln(version);
  debugln(F("check for updates at "
            "https://somepi.ddns.net/gitea/gilex-dev/ESP8266-IOT-timer/\n"));
  load_EEPROM();
  // WLAN-config
  // ---------------------------------------------------------------------------------------------------------------------
  WiFi.mode(WIFI_STA);
  WiFi.config(staticIP, gateway, subnet, dns);
  WiFi.hostname("ESP8266 IOT development");
  WiFi.begin(SSID, PASSWORD);

  Serial.println(F("Connecting ..."));
  while (WiFi.status() != WL_CONNECTED) {
    delay(250);
    Serial.print('.');
  }
  Serial.print(F("\nConnected to "));
  Serial.println(WiFi.SSID());
  Serial.print(F("IP address:\t"));
  Serial.println(WiFi.localIP());
  //----------------------------------------------------------------------------------------------------------------------------------
  myTime.setLocation("de");
  waitForSync();

  debug(F("Local time: "));
  debugln(myTime.dateTime("H:i:s"));
  setInterval();
  server.on("/", HTTP_GET, handleHome);
  server.on("/settings", HTTP_GET, handleSettings);
  server.on("/settings", HTTP_POST, handleTime);
  server.on("/toggle", HTTP_GET, handleToggle);
  server.on("/version", HTTP_GET,
            [] { server.send(200, "text/html", version); });
  server.begin();
  debugln(F("Webserver started"));
}

void loop() {
  server.handleClient();
  timer();
  r++;
  if (r == 7) {
    r = 0;
  }
  yield();
  delay(50);
}
